@@ -4,38 +4,53 @@ import datetime
 import operator
 from decimal import Decimal
 
+from sql import Column
+
+from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, MatchMixin, ValueMixin, DeactivableMixin, fields,
-    sequence_ordered, tree)
+    Model, ModelView, ModelSQL, MatchMixin, ValueMixin, DeactivableMixin,
+    fields, sequence_ordered, tree)
 from trytond import backend
+from trytond.cache import Cache
 from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.tools import grouped_slice
 from trytond.tools.multivalue import migrate_property
 
-__all__ = ['Location', 'Party', 'PartyLocation', 'ProductsByLocationsContext',
-    'LocationLeadTime']
+from trytond.modules.product import price_digits, round_price
 
-STATES = {
-    'readonly': ~Eval('active'),
-}
-DEPENDS = ['active']
+from .exceptions import LocationValidationError
+
+
+class WarehouseWasteLocation(ModelSQL):
+    "Warehouse Waste Location"
+    __name__ = 'stock.location.waste'
+
+    warehouse = fields.Many2One(
+        'stock.location', "Warehouse", required=True, ondelete='CASCADE',
+        domain=[('type', '=', 'warehouse')])
+    location = fields.Many2One(
+        'stock.location', "Waste Location", required=True, ondelete='CASCADE',
+        domain=[('type', '=', 'lost_found')])
 
 
 class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
     "Stock Location"
     __name__ = 'stock.location'
-    name = fields.Char("Name", size=None, required=True, states=STATES,
-        depends=DEPENDS, translate=True)
-    code = fields.Char("Code", size=None, states=STATES, depends=DEPENDS,
-        select=True)
-    address = fields.Many2One("party.address", "Address",
+    _default_warehouse_cache = Cache('stock.location.default_warehouse',
+        context=False)
+
+    name = fields.Char("Name", size=None, required=True, translate=True)
+    code = fields.Char(
+        "Code", size=None, select=True,
+        help="The internal identifier used for the location.")
+    address = fields.Many2One(
+        'party.address', "Address",
         states={
             'invisible': Eval('type') != 'warehouse',
-            'readonly': ~Eval('active'),
             },
-        depends=['type', 'active'])
+        depends=['type'])
     type = fields.Selection([
         ('supplier', 'Supplier'),
         ('customer', 'Customer'),
@@ -45,25 +60,28 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
         ('production', 'Production'),
         ('drop', 'Drop'),
         ('view', 'View'),
-        ], 'Location type', states=STATES, depends=DEPENDS)
+        ], "Location type")
+    type_string = type.translated('type')
     parent = fields.Many2One("stock.location", "Parent", select=True,
         left="left", right="right",
         states={
             'invisible': Eval('type') == 'warehouse',
             },
-        depends=['type'])
+        depends=['type'],
+        help="Used to add structure above the location.")
     left = fields.Integer('Left', required=True, select=True)
     right = fields.Integer('Right', required=True, select=True)
-    childs = fields.One2Many("stock.location", "parent", "Children")
+    childs = fields.One2Many("stock.location", "parent", "Children",
+        help="Used to add structure below the location.")
     flat_childs = fields.Boolean(
         "Flat Children",
-        help="Check to restrict to one level of children.")
+        help="Check to enforce a single level of children with no "
+        "grandchildren.")
     warehouse = fields.Function(fields.Many2One('stock.location', 'Warehouse'),
         'get_warehouse')
     input_location = fields.Many2One(
         "stock.location", "Input", states={
             'invisible': Eval('type') != 'warehouse',
-            'readonly': ~Eval('active'),
             'required': Eval('type') == 'warehouse',
             },
         domain=[
@@ -73,11 +91,11 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
                 ('parent', '=', None),
                 ],
             ],
-        depends=['type', 'active', 'id'])
+        depends=['type', 'id'],
+        help="Where incoming stock is received.")
     output_location = fields.Many2One(
         "stock.location", "Output", states={
             'invisible': Eval('type') != 'warehouse',
-            'readonly': ~Eval('active'),
             'required': Eval('type') == 'warehouse',
         },
         domain=[
@@ -85,11 +103,11 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
             ['OR',
                 ('parent', 'child_of', [Eval('id')]),
                 ('parent', '=', None)]],
-        depends=['type', 'active', 'id'])
+        depends=['type', 'id'],
+        help="Where outgoing stock is sent from.")
     storage_location = fields.Many2One(
         "stock.location", "Storage", states={
             'invisible': Eval('type') != 'warehouse',
-            'readonly': ~Eval('active'),
             'required': Eval('type') == 'warehouse',
         },
         domain=[
@@ -97,40 +115,79 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
             ['OR',
                 ('parent', 'child_of', [Eval('id')]),
                 ('parent', '=', None)]],
-        depends=['type', 'active', 'id'])
+        depends=['type', 'id'],
+        help="The top level location where stock is stored.")
     picking_location = fields.Many2One(
         'stock.location', 'Picking', states={
             'invisible': Eval('type') != 'warehouse',
-            'readonly': ~Eval('active'),
             },
         domain=[
             ('type', '=', 'storage'),
             ('parent', 'child_of', [Eval('storage_location', -1)]),
             ],
-        depends=['type', 'active', 'storage_location'],
-        help='If empty the Storage is used')
+        depends=['type', 'storage_location'],
+        help="Where stock is picked from.\n"
+        "Leave empty to use the storage location.")
+    lost_found_location = fields.Many2One(
+        'stock.location', "Lost and Found",
+        states={
+            'invisible': Eval('type') != 'warehouse',
+            'readonly': ~Eval('active'),
+            },
+        domain=[
+            ('type', '=', 'lost_found'),
+            ],
+        depends=['type', 'active'],
+        help="Used, by inventories, when correcting stock levels "
+        "in the warehouse.")
+    waste_locations = fields.Many2Many(
+        'stock.location.waste', 'warehouse', 'location', "Waste Locations",
+        states={
+            'invisible': Eval('type') != 'warehouse',
+            },
+        domain=[
+            ('type', '=', 'lost_found'),
+            ],
+        depends=['type'],
+        help="The locations used for waste products from the warehouse.")
+    waste_warehouses = fields.Many2Many(
+        'stock.location.waste', 'location', 'warehouse', "Waste Warehouses",
+        states={
+            'invisible': Eval('type') != 'lost_found',
+            },
+        domain=[
+            ('type', '=', 'warehouse'),
+            ],
+        depends=['type'],
+        help="The warehouses that use the location for waste products.")
+
     quantity = fields.Function(
-        fields.Float('Quantity'), 'get_quantity', searcher='search_quantity')
+        fields.Float(
+            "Quantity", digits=(16, Eval('quantity_uom_digits', 2)),
+            depends=['quantity_uom_digits'],
+            help="The amount of stock in the location."),
+        'get_quantity', searcher='search_quantity')
     forecast_quantity = fields.Function(
-        fields.Float('Forecast Quantity'), 'get_quantity',
-        searcher='search_quantity')
-    cost_value = fields.Function(fields.Numeric('Cost Value'),
+        fields.Float(
+            "Forecast Quantity", digits=(16, Eval('quantity_uom_digits', 2)),
+            depends=['quantity_uom_digits'],
+            help="The amount of stock expected to be in the location."),
+        'get_quantity', searcher='search_quantity')
+    quantity_uom = fields.Function(fields.Many2One(
+            'product.uom', "Quantity UOM"),
+        'get_quantity_uom')
+    quantity_uom_digits = fields.Function(fields.Integer(
+            "Quantity UOM Digits"),
+        'get_quantity_uom')
+    cost_value = fields.Function(fields.Numeric(
+            "Cost Value", digits=price_digits,
+            help="The value of the stock in the location."),
         'get_cost_value')
 
     @classmethod
     def __setup__(cls):
         super(Location, cls).__setup__()
         cls._order.insert(0, ('name', 'ASC'))
-        cls._error_messages.update({
-                'invalid_type_for_moves': ('Location "%s" with existing moves '
-                    'cannot be changed to a type that does not support moves.'
-                    ),
-                'child_of_warehouse': ('Location "%(location)s" must be a '
-                    'child of warehouse "%(warehouse)s".'),
-                'inactive_location_with_moves': (
-                    "The location '%(location)s' must be empty "
-                    "to be deactivated."),
-                })
 
         parent_domain = [
             ['OR',
@@ -208,8 +265,10 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
                         ('state', 'not in', ['staging', 'draft']),
                         ])
             if moves:
-                self.raise_user_error(
-                    'invalid_type_for_moves', (self.rec_name,))
+                raise LocationValidationError(
+                    gettext('stock.msg_location_invalid_type_for_moves',
+                        location=self.rec_name,
+                        type=self.type_string))
 
     @classmethod
     def check_inactive(cls, locations):
@@ -218,9 +277,9 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
         empty = cls.get_empty_locations(locations)
         non_empty = set(locations) - set(empty)
         if non_empty:
-            cls.raise_user_error('inactive_location_with_moves', {
-                    'location': next(iter(non_empty)).rec_name,
-                    })
+            raise LocationValidationError(
+                gettext('stock.msg_location_inactive_not_empty',
+                    location=next(iter(non_empty)).rec_name))
 
     @classmethod
     def get_empty_locations(cls, locations=None):
@@ -246,7 +305,7 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
             for sub_ids in grouped_slice(list(empty)):
                 sub_ids = list(sub_ids)
                 moves = Move.search([
-                        ('state', 'not in', ['done', 'cancel']),
+                        ('state', 'not in', ['done', 'cancelled']),
                         ['OR',
                             ('from_location', 'in', sub_ids),
                             ('to_location', 'in', sub_ids),
@@ -287,6 +346,29 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
         if locations:
             return locations[0].id
 
+    @classmethod
+    def get_default_warehouse(cls):
+        warehouse = Transaction().context.get('warehouse')
+        if warehouse:
+            return warehouse
+
+        warehouse = cls._default_warehouse_cache.get(None, -1)
+        if warehouse == -1:
+            warehouses = cls.search([
+                    ('type', '=', 'warehouse'),
+                    ], limit=2)
+            if len(warehouses) == 1:
+                warehouse = warehouses[0].id
+            else:
+                warehouse = None
+            cls._default_warehouse_cache.set(None, warehouse)
+        return warehouse
+
+    @property
+    def lost_found_used(self):
+        if self.warehouse:
+            return self.warehouse.lost_found_location
+
     @classmethod
     def search_rec_name(cls, name, clause):
         if clause[1].startswith('!') or clause[1].startswith('not '):
@@ -298,6 +380,20 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
             ('code',) + tuple(clause[1:]),
             ]
 
+    @classmethod
+    def _get_quantity_grouping(cls):
+        context = Transaction().context
+        grouping, grouping_filter, key = (), (), []
+        if context.get('product') is not None:
+            grouping = ('product',)
+            grouping_filter = ([context['product']],)
+            key = (context['product'],)
+        elif context.get('product_template') is not None:
+            grouping = ('product.template',)
+            grouping_filter = ([context['product_template']],)
+            key = (context['product_template'],)
+        return grouping, grouping_filter, key
+
     @classmethod
     def get_quantity(cls, locations, name):
         pool = Pool()
@@ -309,9 +405,6 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
             return (trans_context.get(name) is not None
                 and isinstance(trans_context[name], int))
 
-        if not any(map(valid_context, ['product', 'product_template'])):
-            return {l.id: None for l in locations}
-
         context = {}
         if (name == 'quantity'
                 and (trans_context.get('stock_date_end', datetime.date.max)
@@ -323,14 +416,10 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
             if not trans_context.get('stock_date_end'):
                 context['stock_date_end'] = datetime.date.max
 
-        if trans_context.get('product') is not None:
-            grouping = ('product',)
-            grouping_filter = ([trans_context['product']],)
-            key = trans_context['product']
-        else:
-            grouping = ('product.template',)
-            grouping_filter = ([trans_context['product_template']],)
-            key = trans_context['product_template']
+        grouping, grouping_filter, key = cls._get_quantity_grouping()
+        if not grouping:
+            return {loc.id: None for loc in locations}
+
         pbl = {}
         for sub_locations in grouped_slice(locations):
             location_ids = [l.id for l in sub_locations]
@@ -341,7 +430,7 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
                         grouping_filter=grouping_filter,
                         with_childs=trans_context.get('with_childs', True)))
 
-        return dict((loc.id, pbl.get((loc.id, key), 0)) for loc in locations)
+        return dict((loc.id, pbl.get((loc.id,) + key, 0)) for loc in locations)
 
     @classmethod
     def search_quantity(cls, name, domain):
@@ -363,6 +452,27 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
                 ids.append(location.id)
         return [('id', 'in', ids)]
 
+    @classmethod
+    def get_quantity_uom(cls, locations, name):
+        pool = Pool()
+        Product = pool.get('product.product')
+        Template = pool.get('product.template')
+        context = Transaction().context
+        value = None
+        uom = None
+        if context.get('product') is not None:
+            product = Product(context['product'])
+            uom = product.default_uom
+        elif context.get('product_template') is not None:
+            template = Template(context['product_template'])
+            uom = template.default_uom
+        if uom:
+            if name == 'quantity_uom':
+                value = uom.id
+            elif name == 'quantity_uom_digits':
+                value = uom.digits
+        return {l.id: value for l in locations}
+
     @classmethod
     def get_cost_value(cls, locations, name):
         pool = Pool()
@@ -398,7 +508,7 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
         # The template may have more than one product
         if cost_price is not None:
             for location in locations:
-                cost_values[location.id] = (
+                cost_values[location.id] = round_price(
                     Decimal(str(location.quantity)) * cost_price)
         return cost_values
 
@@ -428,6 +538,7 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
     def create(cls, vlist):
         locations = super(Location, cls).create(vlist)
         cls._set_warehouse_parent(locations)
+        cls._default_warehouse_cache.clear()
         return locations
 
     @classmethod
@@ -435,6 +546,7 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
         super(Location, cls).write(*args)
         locations = sum(args[::2], [])
         cls._set_warehouse_parent(locations)
+        cls._default_warehouse_cache.clear()
 
         ids = [l.id for l in locations]
         warehouses = cls.search([
@@ -456,10 +568,15 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
                             ('parent', 'child_of', warehouse.id),
                             ]))
                 if location not in childs:
-                    cls.raise_user_error('child_of_warehouse', {
-                            'location': location.rec_name,
-                            'warehouse': warehouse.rec_name,
-                            })
+                    raise LocationValidationError(
+                        gettext('stock.msg_location_child_of_warehouse',
+                            location=location.rec_name,
+                            warehouse=warehouse.rec_name))
+
+    @classmethod
+    def delete(cls, *args):
+        super().delete(*args)
+        cls._default_warehouse_cache.clear()
 
     @classmethod
     def copy(cls, locations, default=None):
@@ -511,14 +628,28 @@ class Location(DeactivableMixin, tree(), ModelSQL, ModelView):
             res.append(new_location)
         return res
 
+    @classmethod
+    def view_attributes(cls):
+        storage_types = Eval('type').in_(['storage', 'warehouse', 'view'])
+        return super().view_attributes() + [
+            ('/tree/field[@name="quantity"]',
+                'visual', If(
+                    storage_types & (Eval('quantity', 0) < 0), 'danger', ''),
+                ['type']),
+            ('/tree/field[@name="forecast_quantity"]',
+                'visual', If(
+                    storage_types & (Eval('forecast_quantity', 0) < 0),
+                    'warning', ''),
+                ['type']),
+            ]
+
 
 supplier_location = fields.Many2One(
     'stock.location', "Supplier Location", domain=[('type', '=', 'supplier')],
-    help='The default source location when receiving products from the party.')
+    help="The default source location for stock received from the party.")
 customer_location = fields.Many2One(
     'stock.location', "Customer Location", domain=[('type', '=', 'customer')],
-    help='The default destination location when sending products to the party.'
-    )
+    help="The default destination location for stock sent to the party.")
 
 
 class Party(metaclass=PoolMeta):
@@ -556,8 +687,7 @@ class PartyLocation(ModelSQL, ValueMixin):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        exist = TableHandler.table_exist(cls._table)
+        exist = backend.TableHandler.table_exist(cls._table)
 
         super(PartyLocation, cls).__register__(module_name)
 
@@ -595,10 +725,10 @@ class ProductsByLocationsContext(ModelView):
     'Products by Locations'
     __name__ = 'stock.products_by_locations.context'
     forecast_date = fields.Date(
-        'At Date', help=('Allow to compute expected '
-            'stock quantities for this date.\n'
-            '* An empty value is an infinite date in the future.\n'
-            '* A date in the past will provide historical values.'))
+        'At Date',
+        help="The date for which the stock quantity is calculated.\n"
+        "* An empty value calculates as far ahead as possible.\n"
+        "* A date in the past will provide historical values.")
     stock_date_end = fields.Function(fields.Date('At Date'),
         'on_change_with_stock_date_end')
 
@@ -614,6 +744,65 @@ class ProductsByLocationsContext(ModelView):
         return self.forecast_date
 
 
+class ProductsByLocations(DeactivableMixin, ModelSQL, ModelView):
+    "Products by Locations"
+    __name__ = 'stock.products_by_locations'
+
+    product = fields.Many2One('product.product', "Product")
+    quantity = fields.Function(
+        fields.Float(
+            "Quantity", digits=(16, Eval('default_uom_digits', 2)),
+            depends=['default_uom_digits']),
+        'get_product', searcher='search_product')
+    forecast_quantity = fields.Function(
+        fields.Float(
+            "Forecast Quantity", digits=(16, Eval('default_uom_digits', 2)),
+            depends=['default_uom_digits']),
+        'get_product', searcher='search_product')
+    default_uom = fields.Function(
+        fields.Many2One('product.uom', "Default UOM"),
+        'get_product', searcher='search_product')
+    default_uom_digits = fields.Function(
+        fields.Integer("Default UOM Digits"), 'get_product')
+    cost_value = fields.Function(
+        fields.Numeric("Cost Value"), 'get_product')
+    consumable = fields.Function(
+        fields.Boolean("Consumable"), 'get_product',
+        searcher='search_product')
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls._order.insert(0, ('product', 'ASC'))
+
+    @classmethod
+    def table_query(cls):
+        pool = Pool()
+        Product = pool.get('product.product')
+        product = Product.__table__()
+        columns = []
+        for fname, field in cls._fields.items():
+            if not hasattr(field, 'set'):
+                if (isinstance(field, fields.Many2One)
+                        and field.get_target() == Product):
+                    column = Column(product, 'id')
+                else:
+                    column = Column(product, fname)
+                columns.append(column.as_(fname))
+        return product.select(*columns)
+
+    def get_product(self, name):
+        value = getattr(self.product, name)
+        if isinstance(value, Model):
+            value = value.id
+        return value
+
+    @classmethod
+    def search_product(cls, name, clause):
+        nested = clause[0].lstrip(name)
+        return [('product.' + name + nested,) + tuple(clause[1:])]
+
+
 class LocationLeadTime(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
     'Location Lead Time'
     __name__ = 'stock.location.lead_time'
@@ -628,7 +817,8 @@ class LocationLeadTime(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
         domain=[
             ('type', '=', 'warehouse'),
             ])
-    lead_time = fields.TimeDelta('Lead Time')
+    lead_time = fields.TimeDelta('Lead Time',
+        help="The time it takes to move stock between the warehouses.")
 
     @classmethod
     def get_lead_time(cls, pattern):
