@@ -16,10 +16,12 @@ from trytond.transaction import Transaction
 from trytond.exceptions import UserWarning, UserError
 from trytond.pool import Pool
 
-from trytond.modules.company.tests import create_company, set_company
+from trytond.modules.company.tests import (
+    create_company, set_company, PartyCompanyCheckEraseMixin, CompanyTestMixin)
 
 
-class StockTestCase(ModuleTestCase):
+class StockTestCase(
+        PartyCompanyCheckEraseMixin, CompanyTestMixin, ModuleTestCase):
     'Test Stock module'
     module = 'stock'
     longMessage = True
@@ -104,6 +106,7 @@ class StockTestCase(ModuleTestCase):
         supplier, = Location.search([('code', '=', 'SUP')])
         customer, = Location.search([('code', '=', 'CUS')])
         storage, = Location.search([('code', '=', 'STO')])
+        storage_empty, = Location.copy([storage])
         company = create_company()
         currency = company.currency
         with set_company(company):
@@ -175,7 +178,7 @@ class StockTestCase(ModuleTestCase):
             Move.do([moves[0], moves[2]])
 
             products_by_location = partial(Product.products_by_location,
-                    [storage.id], [product.id])
+                    [storage.id], grouping_filter=([product.id],))
 
             tests = [
                 ({'stock_date_end': today + relativedelta(days=-6),
@@ -231,73 +234,86 @@ class StockTestCase(ModuleTestCase):
                 'forecast': True,
                 }, 6),
             ]
+            today_quantity = 4
 
             def tests_product_quantity(context, quantity):
-                with transaction.set_context(locations=[storage.id]):
-                    product_reloaded = Product(product.id)
-                    if (not context.get('stock_date_end')
-                            or context['stock_date_end'] > today
-                            or context.get('forecast')):
-                        self.assertEqual(product_reloaded.forecast_quantity,
-                            quantity)
-                    else:
-                        self.assertEqual(product_reloaded.quantity, quantity)
+                product_reloaded = Product(product.id)
+                if (not context.get('stock_date_end')
+                        or context['stock_date_end'] > today):
+                    self.assertEqual(
+                        product_reloaded.forecast_quantity, quantity,
+                        msg='context %r' % context)
+                    self.assertEqual(
+                        product_reloaded.quantity, today_quantity,
+                        msg='context %r' % context)
+                elif context.get('forecast'):
+                    self.assertEqual(
+                        product_reloaded.forecast_quantity, quantity,
+                        msg='context %r' % context)
+                elif context.get('stock_date_end') == today:
+                    self.assertEqual(product_reloaded.quantity, quantity,
+                        msg='context %r' % context)
+                else:
+                    self.assertEqual(
+                        product_reloaded.forecast_quantity, quantity,
+                        msg='context %r' % context)
+                    self.assertEqual(product_reloaded.quantity, quantity,
+                        msg='context %r' % context)
 
             def tests_product_search_quantity(context, quantity):
-                with transaction.set_context(locations=[storage.id]):
-                    if (not context.get('stock_date_end')
-                            or context['stock_date_end'] > today
-                            or context.get('forecast')):
-                        fname = 'forecast_quantity'
-                    else:
-                        fname = 'quantity'
-                    found_products = Product.search([
-                            (fname, '=', quantity),
-                            ])
-                    self.assertIn(product, found_products)
+                if (not context.get('stock_date_end')
+                        or context['stock_date_end'] > today
+                        or context.get('forecast')):
+                    fname = 'forecast_quantity'
+                else:
+                    fname = 'quantity'
+                found_products = Product.search([
+                        (fname, '=', quantity),
+                        ])
+                self.assertIn(product, found_products)
 
-                    found_products = Product.search([
-                            (fname, '!=', quantity),
-                            ])
-                    self.assertNotIn(product, found_products)
+                found_products = Product.search([
+                        (fname, '!=', quantity),
+                        ])
+                self.assertNotIn(product, found_products)
 
-                    found_products = Product.search([
-                            (fname, 'in', (quantity, quantity + 1)),
-                            ])
-                    self.assertIn(product, found_products)
+                found_products = Product.search([
+                        (fname, 'in', (quantity, quantity + 1)),
+                        ])
+                self.assertIn(product, found_products)
 
-                    found_products = Product.search([
-                            (fname, 'not in', (quantity, quantity + 1)),
-                            ])
-                    self.assertNotIn(product, found_products)
+                found_products = Product.search([
+                        (fname, 'not in', (quantity, quantity + 1)),
+                        ])
+                self.assertNotIn(product, found_products)
 
-                    found_products = Product.search([
-                            (fname, '<', quantity),
-                            ])
-                    self.assertNotIn(product, found_products)
-                    found_products = Product.search([
-                            (fname, '<', quantity + 1),
-                            ])
-                    self.assertIn(product, found_products)
+                found_products = Product.search([
+                        (fname, '<', quantity),
+                        ])
+                self.assertNotIn(product, found_products)
+                found_products = Product.search([
+                        (fname, '<', quantity + 1),
+                        ])
+                self.assertIn(product, found_products)
 
-                    found_products = Product.search([
-                            (fname, '>', quantity),
-                            ])
-                    self.assertNotIn(product, found_products)
-                    found_products = Product.search([
-                            (fname, '>', quantity - 1),
-                            ])
-                    self.assertIn(product, found_products)
+                found_products = Product.search([
+                        (fname, '>', quantity),
+                        ])
+                self.assertNotIn(product, found_products)
+                found_products = Product.search([
+                        (fname, '>', quantity - 1),
+                        ])
+                self.assertIn(product, found_products)
 
-                    found_products = Product.search([
-                            (fname, '>=', quantity),
-                            ])
-                    self.assertIn(product, found_products)
+                found_products = Product.search([
+                        (fname, '>=', quantity),
+                        ])
+                self.assertIn(product, found_products)
 
-                    found_products = Product.search([
-                            (fname, '<=', quantity),
-                            ])
-                    self.assertIn(product, found_products)
+                found_products = Product.search([
+                        (fname, '<=', quantity),
+                        ])
+                self.assertIn(product, found_products)
 
             def test_products_by_location():
                 for context, quantity in tests:
@@ -307,8 +323,16 @@ class StockTestCase(ModuleTestCase):
                         else:
                             self.assertEqual(products_by_location(),
                                     {(storage.id, product.id): quantity})
-                            tests_product_quantity(context, quantity)
-                            tests_product_search_quantity(context, quantity)
+                            with transaction.set_context(
+                                    locations=[storage.id]):
+                                tests_product_quantity(context, quantity)
+                                tests_product_search_quantity(
+                                    context, quantity)
+                            with transaction.set_context(
+                                    locations=[storage.id, storage_empty.id]):
+                                tests_product_quantity(context, quantity)
+                                tests_product_search_quantity(
+                                    context, quantity)
 
             test_products_by_location()
 
@@ -411,11 +435,15 @@ class StockTestCase(ModuleTestCase):
             Move.do(moves)
 
             products_by_location = Product.products_by_location(
-                [warehouse.id], [product.id], with_childs=True)
+                [warehouse.id],
+                grouping_filter=([product.id],),
+                with_childs=True)
             self.assertEqual(products_by_location[(warehouse.id, product.id)],
                 1)
             products_by_location = Product.products_by_location(
-                [warehouse.id, storage.id], [product.id], with_childs=True)
+                [warehouse.id, storage.id],
+                grouping_filter=([product.id],),
+                with_childs=True)
             self.assertEqual(
                 products_by_location[(warehouse.id, product.id)], 1)
             self.assertEqual(products_by_location[(storage.id, product.id)], 2)
@@ -428,9 +456,11 @@ class StockTestCase(ModuleTestCase):
 
             with Transaction().set_context(stock_skip_warehouse=True):
                 products_by_location = Product.products_by_location(
-                    [warehouse.id], [product.id], with_childs=True)
+                    [warehouse.id],
+                    grouping_filter=([product.id],),
+                    with_childs=True)
                 products_by_location_all = Product.products_by_location(
-                    [warehouse.id], None, with_childs=True)
+                    [warehouse.id], with_childs=True)
                 self.assertEqual(
                     products_by_location[(warehouse.id, product.id)], 2)
                 self.assertEqual(
@@ -523,14 +553,16 @@ class StockTestCase(ModuleTestCase):
 
             # Test flat location
             products_by_location = Product.products_by_location(
-                [storage.id], [product.id], with_childs=True)
+                [storage.id],
+                grouping_filter=([product.id],),
+                with_childs=True)
             self.assertEqual(
                 products_by_location[(storage.id, product.id)], 2)
 
             # Test mixed flat and nested location
             products_by_location = Product.products_by_location(
                 [storage.parent.id, storage.id, storage1.id, storage2.id],
-                [product.id],
+                grouping_filter=([product.id],),
                 with_childs=True)
             self.assertEqual(
                 products_by_location[(storage.parent.id, product.id)], 2)
@@ -543,7 +575,9 @@ class StockTestCase(ModuleTestCase):
 
             # Test non flat
             products_by_location = Product.products_by_location(
-                [lost_found.id], [product.id], with_childs=True)
+                [lost_found.id],
+                grouping_filter=([product.id],),
+                with_childs=True)
             self.assertEqual(
                 products_by_location[(lost_found.id, product.id)], -2)
 
@@ -551,6 +585,165 @@ class StockTestCase(ModuleTestCase):
         "Test products_by_location on flat_childs with period closed"
         self.test_products_by_location_flat_childs(period_closed=True)
 
+    @with_transaction()
+    def test_products_by_location_2nd_level_flat_childs(self):
+        "Test products_by_location on 2nd level flat_childs"
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+        Date = pool.get('ir.date')
+
+        unit, = Uom.search([('name', '=', 'Unit')])
+        template, = Template.create([{
+                    'name': "Product",
+                    'type': 'goods',
+                    'list_price': Decimal(0),
+                    'default_uom': unit.id,
+                    }])
+        product, = Product.create([{
+                    'template': template.id,
+                    }])
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        storage1, = Location.create([{
+                    'name': 'Storage 1',
+                    'type': 'storage',
+                    'flat_childs': True,
+                    'parent': storage.id,
+                    }])
+        storage2, = Location.create([{
+                    'name': 'Storage 2',
+                    'type': 'storage',
+                    'parent': storage1.id,
+                    }])
+
+        company = create_company()
+        with set_company(company):
+            date = Date.today() - relativedelta(days=1)
+
+            moves = Move.create([{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 80,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'effective_date': date,
+                        'company': company.id,
+                        'unit_price': 10.0
+                        }, {
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 60,
+                        'from_location': storage.id,
+                        'to_location': storage2.id,
+                        'effective_date': date,
+                        'company': company.id,
+                        }])
+            Move.do(moves)
+
+            # Test 2nd level location
+            products_by_location = Product.products_by_location(
+                [storage.id],
+                grouping_filter=([product.id],),
+                with_childs=True)
+            self.assertEqual(
+                products_by_location[(storage.id, product.id)], 80)
+
+            # Test 1st level and 2nd level nested locations
+            products_by_location = Product.products_by_location(
+                [storage.id, storage1.id],
+                grouping_filter=([product.id],),
+                with_childs=True)
+            self.assertEqual(
+                products_by_location[(storage.id, product.id)], 80)
+            self.assertEqual(
+                products_by_location[(storage1.id, product.id)], 60)
+
+            # Test mixed flat and 2nd level nested locations
+            products_by_location = Product.products_by_location(
+                [storage.id, storage2.id],
+                grouping_filter=([product.id],),
+                with_childs=True)
+            self.assertEqual(
+                products_by_location[(storage.id, product.id)], 80)
+            self.assertEqual(
+                products_by_location[(storage2.id, product.id)], 60)
+
+    @with_transaction()
+    def test_templates_by_location(self, period_closed=False):
+        "Test products_by_location grouped by template"
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+        Period = pool.get('stock.period')
+        Date = pool.get('ir.date')
+
+        unit, = Uom.search([('name', '=', 'Unit')])
+        template, = Template.create([{
+                    'name': 'Template',
+                    'type': 'goods',
+                    'list_price': Decimal(0),
+                    'default_uom': unit.id,
+                    }])
+        product1, product2 = Product.create([{
+                    'template': template.id,
+                    }] * 2)
+
+        lost_found, = Location.search([('type', '=', 'lost_found')])
+        storage, = Location.search([('code', '=', 'STO')])
+        input_, = Location.search([('code', '=', 'IN')])
+        company = create_company()
+        with set_company(company):
+            date = Date.today() - relativedelta(days=1)
+
+            moves = Move.create([{
+                        'product': product1.id,
+                        'uom': unit.id,
+                        'quantity': 2,
+                        'from_location': lost_found.id,
+                        'to_location': storage.id,
+                        'planned_date': date,
+                        'effective_date': date,
+                        'company': company.id,
+                        }, {
+                        'product': product2.id,
+                        'uom': unit.id,
+                        'quantity': 3,
+                        'from_location': input_.id,
+                        'to_location': storage.id,
+                        'planned_date': date,
+                        'effective_date': date,
+                        'company': company.id,
+                        }])
+            Move.do(moves)
+
+            if period_closed:
+                period, = Period.create([{
+                            'date': date,
+                            'company': company.id,
+                            }])
+                Period.close([period])
+
+            templates_by_location = Product.products_by_location(
+                [storage.id],
+                grouping=('product.template',),
+                grouping_filter=([template.id],))
+
+            self.assertDictEqual(templates_by_location, {
+                    (storage.id, template.id): 5,
+                    })
+
+    def test_templates_by_location_period_closed(self):
+        "Test products_by_location grouped by template with period closed"
+        self.test_templates_by_location(period_closed=True)
+
     @with_transaction()
     def test_period(self):
         'Test period'
@@ -642,7 +835,7 @@ class StockTestCase(ModuleTestCase):
             ]
 
             products_by_location = partial(Product.products_by_location,
-                [storage.id], [product.id])
+                [storage.id], grouping_filter=([product.id],))
 
             tests_pbl = [
                 ({'stock_date_end': today + relativedelta(days=-6)}, 0),
@@ -650,6 +843,7 @@ class StockTestCase(ModuleTestCase):
                 ({'stock_date_end': today + relativedelta(days=-4)}, 25),
                 ({'stock_date_end': today + relativedelta(days=-3)}, 20),
                 ({'stock_date_end': today + relativedelta(days=-2)}, 20),
+                ({'stock_date_end': today + relativedelta(days=-1)}, 20),
                 ({'stock_date_end': today}, 20),
                 ({'stock_date_end': datetime.date.max}, 23),
                 ]
@@ -708,6 +902,18 @@ class StockTestCase(ModuleTestCase):
                         'unit_price': Decimal('1'),
                         'currency': currency.id,
                         }])
+            self.assertRaises(Exception, Move.create, [{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 10,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'planned_date': today + relativedelta(days=-3),
+                        'effective_date': today + relativedelta(days=-3),
+                        'company': company.id,
+                        'unit_price': Decimal('1'),
+                        'currency': currency.id,
+                        }])
 
             # Test close period check
             period, = Period.create([{
@@ -893,6 +1099,126 @@ class StockTestCase(ModuleTestCase):
             self.assertFalse(Move.assign_try(moves))
             self.assertEqual([m.state for m in moves], ['assigned', 'draft'])
 
+    @with_transaction()
+    def test_assign_try_skip_to_location(self):
+        "Test Move assign_try skip to_location"
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+
+        uom, = Uom.search([('name', '=', 'Meter')])
+        template = Template(
+            name='Test Move.assign_try',
+            type='goods',
+            list_price=Decimal(1),
+            default_uom=uom,
+            )
+        template.save()
+        product = Product(template=template.id)
+        product.save()
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        child, = Location.copy([storage], default={'parent': storage.id})
+
+        company = create_company()
+        with set_company(company):
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': child.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+            Move.do([move])
+
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': storage.id,
+                        'to_location': child.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+
+            self.assertFalse(Move.assign_try([move]))
+            self.assertEqual(move.state, 'draft')
+
+    @with_transaction()
+    def test_assign_try_prefer_from_location(self):
+        "Test Move assign_try prefer from location"
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+
+        uom, = Uom.search([('name', '=', 'Meter')])
+        template = Template(
+            name='Test Move.assign_try',
+            type='goods',
+            list_price=Decimal(1),
+            default_uom=uom,
+            )
+        template.save()
+        product = Product(template=template.id)
+        product.save()
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        # Ensure storage2 comes first when ordering locations by name
+        storage2, = Location.copy(
+            [storage], default={'name': "AAA", 'parent': storage.id})
+        customer, = Location.search([('code', '=', 'CUS')])
+
+        company = create_company()
+        with set_company(company):
+            moves = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }, {
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': storage2.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+            Move.do(moves)
+
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': storage.id,
+                        'to_location': customer.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+
+            Move.assign_try([move])
+
+            self.assertEqual(move.state, 'assigned')
+            self.assertEqual(move.from_location, storage)
+
     @with_transaction()
     def test_assign_without_moves(self):
         "Test Move assign_try with empty moves"
@@ -901,6 +1227,76 @@ class StockTestCase(ModuleTestCase):
 
         self.assertTrue(Move.assign_try([]))
 
+    @with_transaction()
+    def test_assign_try_modified_move(self):
+        "Test Move assign_try with modified move"
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+
+        uom, = Uom.search([('name', '=', 'Meter')])
+        template = Template(
+            name='Test Move.assign_try',
+            type='goods',
+            list_price=Decimal(1),
+            default_uom=uom,
+            )
+        template.save()
+        product = Product(template=template.id)
+        product.save()
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        storage2, = Location.copy([storage])
+        customer, = Location.search([('code', '=', 'CUS')])
+
+        company = create_company()
+        with set_company(company):
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+            Move.do([move])
+
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 2,
+                        'from_location': storage2.id,
+                        'to_location': customer.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+
+            # Assign from a different location
+            move.from_location = storage
+            self.assertFalse(Move.assign_try([move]))
+
+            move, = Move.search([
+                    ('to_location', '=', customer.id),
+                    ('state', '=', 'assigned'),
+                    ])
+            self.assertEqual(move.from_location, storage)
+            self.assertEqual(move.quantity, 1)
+
+            move, = Move.search([
+                    ('to_location', '=', customer.id),
+                    ('state', '=', 'draft'),
+                    ])
+            # Remaining move has the original location
+            self.assertEqual(move.from_location, storage2)
+            self.assertEqual(move.quantity, 1)
+
     @with_transaction()
     def test_products_by_location_assign(self):
         "Test products by location for assignation"
@@ -955,41 +1351,47 @@ class StockTestCase(ModuleTestCase):
             with Transaction().set_context(
                     stock_date_end=today,
                     stock_assign=True):
-                pbl = Product.products_by_location([storage.id], [product.id])
+                pbl = Product.products_by_location(
+                    [storage.id], grouping_filter=([product.id],))
                 self.assertDictEqual(pbl, {})
 
             Move.assign([move_customer])
             with Transaction().set_context(
                     stock_date_end=today,
                     stock_assign=True):
-                pbl = Product.products_by_location([storage.id], [product.id])
+                pbl = Product.products_by_location(
+                    [storage.id], grouping_filter=([product.id],))
                 self.assertDictEqual(pbl, {(storage.id, product.id): -5})
 
             Move.do([move_supplier])
             with Transaction().set_context(
                     stock_date_end=today,
                     stock_assign=True):
-                pbl = Product.products_by_location([storage.id], [product.id])
+                pbl = Product.products_by_location(
+                    [storage.id], grouping_filter=([product.id],))
                 self.assertDictEqual(pbl, {(storage.id, product.id): 5})
 
             with Transaction().set_context(
                     stock_date_end=today + relativedelta(days=1),
                     stock_assign=True):
-                pbl = Product.products_by_location([storage.id], [product.id])
+                pbl = Product.products_by_location(
+                    [storage.id], grouping_filter=([product.id],))
                 self.assertDictEqual(pbl, {(storage.id, product.id): 5})
 
             with Transaction().set_context(
                     stock_date_start=today,
                     stock_date_end=today,
                     stock_assign=True):
-                pbl = Product.products_by_location([storage.id], [product.id])
+                pbl = Product.products_by_location(
+                    [storage.id], grouping_filter=([product.id],))
                 self.assertDictEqual(pbl, {(storage.id, product.id): 10})
 
             with Transaction().set_context(
                     stock_date_start=today + relativedelta(days=1),
                     stock_date_end=today + relativedelta(days=1),
                     stock_assign=True):
-                pbl = Product.products_by_location([storage.id], [product.id])
+                pbl = Product.products_by_location(
+                    [storage.id], grouping_filter=([product.id],))
                 self.assertDictEqual(pbl, {(storage.id, product.id): -5})
 
     @with_transaction()
@@ -1026,7 +1428,6 @@ class StockTestCase(ModuleTestCase):
                     'name': "Product",
                     'type': 'goods',
                     'list_price': Decimal(0),
-                    'cost_price': Decimal(0),
                     'default_uom': unit.id,
                     }])
         product, = Product.create([{'template': template.id}])
@@ -1067,7 +1468,6 @@ class StockTestCase(ModuleTestCase):
                     'name': "Product",
                     'type': 'goods',
                     'list_price': Decimal(0),
-                    'cost_price': Decimal(0),
                     'default_uom': unit.id,
                     }])
         product, = Product.create([{'template': template.id}])
@@ -1097,6 +1497,11 @@ class StockTestCase(ModuleTestCase):
 def suite():
     suite = trytond.tests.test_tryton.suite()
     suite.addTests(unittest.TestLoader().loadTestsFromTestCase(StockTestCase))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_product_quantities_by_warehouse.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     suite.addTests(doctest.DocFileSuite('scenario_stock_shipment_out.rst',
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
@@ -1111,11 +1516,26 @@ def suite():
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_recompute_average_cost_price.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     suite.addTests(doctest.DocFileSuite(
             'scenario_stock_inventory.rst',
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_inventory_empty_quantity.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_inventory_count.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     suite.addTests(doctest.DocFileSuite('scenario_stock_shipment_internal.rst',
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
@@ -1124,4 +1544,29 @@ def suite():
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_shipment_in_same_storage_input.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_shipment_out_return_same_storage_input.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_move_in_future.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_shipment_in_return.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_reporting_margin.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     return suite
